package org.jfw.apt.orm.daoHandler;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import javax.lang.model.element.Element;

import org.jfw.apt.exception.AptException;
import org.jfw.apt.orm.annotation.dao.method.operator.InsertAndGenKey;
import org.jfw.apt.orm.core.ColumnHandlerFactory;
import org.jfw.apt.orm.core.ColumnWriterCache;
import org.jfw.apt.orm.core.model.CalcColumn;
import org.jfw.apt.orm.core.model.Column;
import org.jfw.apt.orm.core.model.DataEntry;
import org.jfw.apt.orm.core.model.DataEntryFactory;
import org.jfw.apt.orm.core.model.Table;

public class InsertAndGenKeyHandler extends BaseDaoHandler {
	protected List<ColumnWriterCache> sqlparams = new ArrayList<ColumnWriterCache>();
	protected Table fromTable;
	private boolean isLong = false;
	private String keyName;

	protected boolean doReplaceSource;

	@Override
	protected void prepare() throws AptException {
		this.sqlparams.clear();
	}

	@Override
	protected void writeMethodContent() throws AptException {
		this.prepareTable();
		this.prepareSqlParams();
		this.prepareSQL();

		if (this.doReplaceSource) {
			cw.l("try{");
		}
		cw.bL("java.sql.PreparedStatement ps = con.prepareStatement(sql");
		cw.el(",1);");
		cw.l("try{");

		this.handleSqlParameters();
		cw.l("ps.executeUpdate();");
		String rsname = cw.getMethodTempVarName();
		cw.bL("java.sql.ResultSet ").w(rsname).el(" =  ps.getGeneratedKeys();");
		cw.l("try{");
		cw.bL("if(").w(rsname).el(".next()){");
		cw.bL("return ").w(rsname).w(".get").w(this.isLong ? "Long" : "Int").w("(\"").w(this.keyName).el("\");");
		cw.l("}else{");
		cw.l("return -1;").l("}");
		cw.l("}finally{").writeClosing(rsname).l("}");
		cw.l("}finally{").writeClosing("ps").l("}");
		if (this.doReplaceSource) {
			cw.l("}finally{");
			this.replaceResource();
			cw.l("}");
		}

	}

	protected void handleSqlParameters() throws AptException {
		for (ListIterator<ColumnWriterCache> it = this.sqlparams.listIterator(); it.hasNext();) {
			it.next().writeValue();
		}
	}

	protected void prepareTable() throws AptException {
		String entryJavaName = null;
			if (this.me.getParams().size() != 2)
				throw new AptException(ref, "method paramters size must be 2 and the second paramter with @" + Table.class.getName());
			entryJavaName = this.me.getParams().get(1).getTypeName();
				this.fromTable = (Table) DataEntryFactory.get(entryJavaName, DataEntry.TABLE);
		if (null == this.fromTable)
			throw new AptException(ref, "class[" + entryJavaName + "] not with @" + org.jfw.apt.orm.annotation.entry.Table.class.getName());

	}



	@Override
	protected void checkJdbc() throws AptException {
		super.checkJdbc();
		InsertAndGenKey ing = this.ref.getAnnotation(InsertAndGenKey.class);
		this.isLong = ing.isLong();
		this.keyName = ing.value();
		String rt = this.me.getReturnType();
		if(isLong){
			if (!"long".equals(rt))
				throw new AptException(ref, "mehtod must return long");
		}else{
			if (!"int".equals(rt))
				throw new AptException(ref, "mehtod must return int");
	}}

	protected void replaceResource() {
		for (ColumnWriterCache cwc : this.sqlparams) {
			if (cwc.isReplaceResource())
				cwc.replaceResource();
		}

	}



	@Override
	public boolean match(Element ele) {
		return null != ele.getAnnotation(InsertAndGenKey.class);
	}

	protected void prepareSqlParams() {
		this.doReplaceSource = false;
		String pn = this.me.getParams().get(1).getName();
		LinkedList<Column> list = new LinkedList<Column>();
		for (ListIterator<CalcColumn> it = this.fromTable.getAllColumn().listIterator(); it.hasNext();) {
			Column col = (Column) it.next();
			if (col.isInsertable()) {
				if (null == col.getFixSqlValueWithInsert()) {
					list.add(col);
				} else {
					list.add(0, col);
				}
			}
		}

		cw.bL("String sql=\"INSERT INTO ").w(this.fromTable.getFromSentence()).w(" (");

		boolean first = true;
		for (ListIterator<Column> it = list.listIterator(); it.hasNext();) {
			if (first)
				first = false;
			else
				cw.w(",");
			Column col = it.next();
			cw.w(col.getSqlName());
		}
		cw.w(") VALUES (");
		first = true;
		for (ListIterator<Column> it = list.listIterator(); it.hasNext();) {
			if (first)
				first = false;
			else
				cw.w(",");
			Column col = it.next();
			if (null == col.getFixSqlValueWithInsert()) {
				cw.w("?");
				ColumnWriterCache cwc = ColumnWriterCache.build(pn + "." + col.getGetter() + "()", true, col.isNullable(), cw,
						ColumnHandlerFactory.get(col.getHandlerClass()));
				cwc.setDynamic(false);
				if (cwc.isReplaceResource())
					this.doReplaceSource = true;
				cwc.setDynamic(false);
				this.sqlparams.add(cwc);
			} else {
				cw.ws(col.getFixSqlValueWithInsert());
			}
		}
		cw.el(")\";");

		for (ColumnWriterCache cwc : this.sqlparams) {
			cwc.prepare();
		}

	}

	protected void prepareSQL() {
	}
}
